@@ -29,23 +29,23 @@
 
 /* Forward declarations */
 
-static void gtk_dial_class_init               (GtkDialClass    *klass);
-static void gtk_dial_init                     (GtkDial         *dial);
+static void gtk_dial_class_init               (GtkDialClass     *klass);
+static void gtk_dial_init                     (GtkDial          *dial);
 static void gtk_dial_destroy                  (GtkObject        *object);
 static void gtk_dial_realize                  (GtkWidget        *widget);
-static void gtk_dial_size_request             (GtkWidget      *widget,
-					       GtkRequisition *requisition);
-static void gtk_dial_size_allocate            (GtkWidget     *widget,
-					       GtkAllocation *allocation);
-static gint gtk_dial_expose                   (GtkWidget        *widget,
-						GdkEventExpose   *event);
-static gint gtk_dial_button_press             (GtkWidget        *widget,
-						GdkEventButton   *event);
-static gint gtk_dial_button_release           (GtkWidget        *widget,
-						GdkEventButton   *event);
-static gint gtk_dial_motion_notify            (GtkWidget        *widget,
-						GdkEventMotion   *event);
-static gint gtk_dial_timer                    (GtkDial         *dial);
+static void gtk_dial_size_request             (GtkWidget        *widget,
+                                               GtkRequisition   *requisition);
+static void gtk_dial_size_allocate            (GtkWidget        *widget,
+                                               GtkAllocation    *allocation);
+static gboolean gtk_dial_expose               (GtkWidget        *widget,
+                                               GdkEventExpose   *event);
+static gboolean gtk_dial_button_press         (GtkWidget        *widget,
+                                               GdkEventButton   *event);
+static gboolean gtk_dial_button_release       (GtkWidget        *widget,
+                                               GdkEventButton   *event);
+static gboolean gtk_dial_motion_notify        (GtkWidget        *widget,
+                                               GdkEventMotion   *event);
+static gboolean gtk_dial_timer                (GtkDial          *dial);
 
 static void gtk_dial_update_mouse             (GtkDial *dial, gint x, gint y);
 static void gtk_dial_update                   (GtkDial *dial);
@@ -65,7 +65,7 @@ gtk_dial_get_type ()
 
   if (!dial_type)
     {
-      static const GTypeInfo dial_info =
+      const GTypeInfo dial_info =
       {
 	sizeof (GtkDialClass),
 	NULL,
@@ -93,7 +93,7 @@ gtk_dial_class_init (GtkDialClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
 
-  parent_class = gtk_type_class (gtk_widget_get_type ());
+  parent_class = g_type_class_peek_parent (class);
 
   object_class->destroy = gtk_dial_destroy;
 
@@ -146,14 +146,13 @@ gtk_dial_destroy (GtkObject *object)
 
   dial = GTK_DIAL (object);
 
-  if (dial->adjustment) 
+  if (dial->adjustment)
     {
       g_object_unref (GTK_OBJECT (dial->adjustment));
       dial->adjustment = NULL;
     }
 
-  if (GTK_OBJECT_CLASS (parent_class)->destroy)
-    (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+  GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
 GtkAdjustment*
@@ -191,11 +190,11 @@ gtk_dial_set_adjustment (GtkDial      *dial,
   dial->adjustment = adjustment;
   g_object_ref (GTK_OBJECT (dial->adjustment));
 
-  g_signal_connect (GTK_OBJECT (adjustment), "changed",
-		    GTK_SIGNAL_FUNC (gtk_dial_adjustment_changed),
+  g_signal_connect (G_OBJECT (adjustment), "changed",
+		    G_CALLBACK (gtk_dial_adjustment_changed),
 		    (gpointer) dial);
-  g_signal_connect (GTK_OBJECT (adjustment), "value_changed",
-		    GTK_SIGNAL_FUNC (gtk_dial_adjustment_value_changed),
+  g_signal_connect (G_OBJECT (adjustment), "value_changed",
+		    G_CALLBACK (gtk_dial_adjustment_value_changed),
 		    (gpointer) dial);
 
   dial->old_value = adjustment->value;
@@ -215,7 +214,7 @@ gtk_dial_realize (GtkWidget *widget)
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_DIAL (widget));
 
-  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
+  gtk_widget_set_realized (widget, TRUE);
   dial = GTK_DIAL (widget);
 
   attributes.x = widget->allocation.x;
@@ -262,7 +261,7 @@ gtk_dial_size_allocate (GtkWidget     *widget,
   widget->allocation = *allocation;
   dial = GTK_DIAL (widget);
 
-  if (GTK_WIDGET_REALIZED (widget))
+  if (gtk_widget_get_realized (widget))
     {
 
       gdk_window_move_resize (widget->window,
@@ -274,9 +273,9 @@ gtk_dial_size_allocate (GtkWidget     *widget,
   dial->pointer_width = dial->radius / 5;
 }
 
-static gint
-gtk_dial_expose (GtkWidget      *widget,
-		 GdkEventExpose *event)
+static gboolean
+gtk_dial_expose( GtkWidget      *widget,
+		 GdkEventExpose *event )
 {
   GtkDial *dial;
   GdkPoint points[6];
@@ -334,6 +333,8 @@ gtk_dial_expose (GtkWidget      *widget,
                 widget->style->bg_gc[GTK_STATE_NORMAL];
   blankstyle->black_gc =
                 widget->style->bg_gc[GTK_STATE_NORMAL];
+  blankstyle->depth =
+                gdk_drawable_get_depth( GDK_DRAWABLE (widget->window));
 
   gtk_paint_polygon (blankstyle,
                     widget->window,
@@ -413,9 +414,9 @@ gtk_dial_expose (GtkWidget      *widget,
   return FALSE;
 }
 
-static gint
-gtk_dial_button_press (GtkWidget      *widget,
-		       GdkEventButton *event)
+static gboolean
+gtk_dial_button_press( GtkWidget      *widget,
+		       GdkEventButton *event )
 {
   GtkDial *dial;
   gint dx, dy;
@@ -457,9 +458,9 @@ gtk_dial_button_press (GtkWidget      *widget,
   return FALSE;
 }
 
-static gint
-gtk_dial_button_release (GtkWidget      *widget,
-			  GdkEventButton *event)
+static gboolean
+gtk_dial_button_release( GtkWidget      *widget,
+                         GdkEventButton *event )
 {
   GtkDial *dial;
 
@@ -476,7 +477,7 @@ gtk_dial_button_release (GtkWidget      *widget,
       dial->button = 0;
 
       if (dial->policy == GTK_UPDATE_DELAYED)
-	gtk_timeout_remove (dial->timer);
+	g_source_remove (dial->timer);
       
       if ((dial->policy != GTK_UPDATE_CONTINUOUS) &&
 	  (dial->old_value != dial->adjustment->value))
@@ -486,9 +487,9 @@ gtk_dial_button_release (GtkWidget      *widget,
   return FALSE;
 }
 
-static gint
-gtk_dial_motion_notify (GtkWidget      *widget,
-			 GdkEventMotion *event)
+static gboolean
+gtk_dial_motion_notify( GtkWidget      *widget,
+                        GdkEventMotion *event )
 {
   GtkDial *dial;
   GdkModifierType mods;
@@ -531,8 +532,8 @@ gtk_dial_motion_notify (GtkWidget      *widget,
   return FALSE;
 }
 
-static gint
-gtk_dial_timer (GtkDial *dial)
+static gboolean
+gtk_dial_timer( GtkDial *dial )
 {
   g_return_val_if_fail (dial != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_DIAL (dial), FALSE);
@@ -544,7 +545,7 @@ gtk_dial_timer (GtkDial *dial)
 }
 
 static void
-gtk_dial_update_mouse (GtkDial *dial, gint x, gint y)
+gtk_dial_update_mouse( GtkDial *dial, gint x, gint y )
 {
   gint xc, yc;
   gfloat old_value;
@@ -583,11 +584,11 @@ gtk_dial_update_mouse (GtkDial *dial, gint x, gint y)
 	  if (dial->policy == GTK_UPDATE_DELAYED)
 	    {
 	      if (dial->timer)
-		gtk_timeout_remove (dial->timer);
+		g_source_remove (dial->timer);
 
-	      dial->timer = gtk_timeout_add (SCROLL_DELAY_LENGTH,
-					     (GtkFunction) gtk_dial_timer,
-					     (gpointer) dial);
+	      dial->timer = gdk_threads_add_timeout (SCROLL_DELAY_LENGTH,
+					   (GSourceFunc) gtk_dial_timer,
+					   (gpointer) dial);
 	    }
 	}
     }
