@@ -222,6 +222,11 @@ gtk_recent_manager_class_init (GtkRecentManagerClass *klass)
    * gtk_recent_manager_get_items() function.
    *
    * Since: 2.10
+   *
+   * Deprecated: 2.22: Setting this property does not have any effect.
+   *   The length of the list should be managed by the view (implementing
+   *   #GtkRecentChooser), and not by the model (the #GtkRecentManager).
+   *   See #GtkRecentChooser:limit.
    */
   g_object_class_install_property (gobject_class,
   				   PROP_LIMIT,
@@ -231,7 +236,7 @@ gtk_recent_manager_class_init (GtkRecentManagerClass *klass)
   				   		     -1,
   				   		     G_MAXINT,
   				   		     DEFAULT_LIMIT,
-  				   		     G_PARAM_READWRITE));
+                                                     G_PARAM_READWRITE | G_PARAM_DEPRECATED));
   /**
    * GtkRecentManager:size
    * 
@@ -702,6 +707,10 @@ gtk_recent_manager_set_screen (GtkRecentManager *manager,
  * items.
  *
  * Since: 2.10
+ *
+ * Deprecated: 2.22: The length of the list should be managed by the
+ *   view (implementing #GtkRecentChooser), and not by the model (the
+ *   #GtkRecentManager). See #GtkRecentChooser:limit.
  */
 void
 gtk_recent_manager_set_limit (GtkRecentManager *manager,
@@ -725,6 +734,10 @@ gtk_recent_manager_set_limit (GtkRecentManager *manager,
  * Return value: the number of items to return, or -1 for every item.
  *
  * Since: 2.10
+ *
+ * Deprecated: 2.22: The length of the list should be managed by the
+ *   view (implementing #GtkRecentChooser), and not by the model (the
+ *   #GtkRecentManager). See #GtkRecentChooser:limit.
  */
 gint
 gtk_recent_manager_get_limit (GtkRecentManager *manager)
@@ -986,7 +999,7 @@ gtk_recent_manager_add_full (GtkRecentManager     *manager,
  * gtk_recent_manager_remove_item:
  * @manager: a #GtkRecentManager
  * @uri: the URI of the item you wish to remove
- * @error: return location for a #GError, or %NULL
+ * @error: (allow-none): return location for a #GError, or %NULL
  *
  * Removes a resource pointed by @uri from the recently used resources
  * list handled by a recent manager.
@@ -1135,7 +1148,7 @@ build_recent_info (GBookmarkFile  *bookmarks,
  * gtk_recent_manager_lookup_item:
  * @manager: a #GtkRecentManager
  * @uri: a URI
- * @error: a return location for a #GError, or %NULL
+ * @error: (allow-none): a return location for a #GError, or %NULL
  *
  * Searches for a URI inside the recently used resources list, and
  * returns a structure containing informations about the resource
@@ -1198,9 +1211,9 @@ gtk_recent_manager_lookup_item (GtkRecentManager  *manager,
  * gtk_recent_manager_move_item:
  * @manager: a #GtkRecentManager
  * @uri: the URI of a recently used resource
- * @new_uri: the new URI of the recently used resource, or %NULL to
+ * @new_uri: (allow-none): the new URI of the recently used resource, or %NULL to
  *    remove the item pointed by @uri in the list
- * @error: a return location for a #GError, or %NULL
+ * @error: (allow-none): a return location for a #GError, or %NULL
  *
  * Changes the location of a recently used resource from @uri to @new_uri.
  * 
@@ -1294,17 +1307,11 @@ gtk_recent_manager_get_items (GtkRecentManager *manager)
   if (!priv->recent_items)
     return NULL;
 
-  if (priv->limit == 0)
-    return NULL;
-  
   uris = g_bookmark_file_get_uris (priv->recent_items, &uris_len);
   for (i = 0; i < uris_len; i++)
     {
       GtkRecentInfo *info;
       
-      if (priv->limit != -1 && i == priv->limit)
-        break;
-      
       info = gtk_recent_info_new (uris[i]);
       build_recent_info (priv->recent_items, info);
       
@@ -1339,7 +1346,7 @@ purge_recent_items_list (GtkRecentManager  *manager,
 /**
  * gtk_recent_manager_purge_items:
  * @manager: a #GtkRecentManager
- * @error: a return location for a #GError, or %NULL
+ * @error: (allow-none): a return location for a #GError, or %NULL
  *
  * Purges every item from the recently used resources list.
  *
